
# q-expansion of newform 603.2.a.f, downloaded from the LMFDB on 05 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 603
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[2], [0], [0], [0], [6], [4], [7], [-5], [1], [-1], [-4], [3], [0], [-6], [-9], [-10], [-3], [2], [-1], [16], [-7], [8], [4], [15], [4], [0], [4], [-7], [4], [-6], [-9], [-4], [-14], [2], [-19], [-21], [17], [-13], [-12], [11], [-26], [23], [24], [-15], [22], [7], [-20], [15], [19], [-18], [16], [-16], [17], [2], [19], [8], [6], [-22], [-2], [18], [-19], [10], [23], [-26], [-18], [14], [-4], [2], [6], [14], [0], [-5], [28], [8], [-10], [-12], [10], [29], [30], [-32], [7], [19], [-5], [20], [-29], [-4], [27], [-35], [21], [28], [-12], [35], [4], [-5], [-30], [8], [-15], [-30], [41], [-40], [-2], [14], [-8], [-15], [-7], [-12], [24], [6], [6], [-45], [8], [-33], [39], [20], [-6], [0], [36], [-18], [-10], [-33], [-12], [0], [30], [14], [-5], [-14], [-10], [1], [2], [34], [44], [-40], [-49], [50], [3], [34], [33], [-4], [-34], [20], [-2], [27], [-17], [21], [45], [-47], [-11], [48], [20], [-3], [-19], [-21], [-20], [37], [37], [9], [42], [20], [8], [10], [-47], [-11], [-48], [-55], [-29], [-56], [46], [30]]
